
#include <Servo.h>
#include <NewPing.h>
#include <Wire.h>
#include <DFRobot_QMC5883.h>

//Servos needed
Servo myservo1;

//Ultrasonic sensor settings
NewPing sonar(A1, A2, 200); //trigger pin, echo pin, and max distance to ping.

//Magnometer(digital compass)
DFRobot_QMC5883 compass;

//Array for storing Ultrasonic values
byte dataArrayUL1[1];
byte dataArrayUL2[1];
byte dataArrayUL3[1];
byte dataArrayUR1[1];
byte dataArrayUR2[1]; 
byte dataArrayUR3[1];  
           
int arrayIndexUL1 = 0;
int arrayIndexUL2 = 0;
int arrayIndexUL3 = 0;
int arrayIndexUR1 = 0;
int arrayIndexUR2 = 0;
int arrayIndexUR3 = 0;

// Orientation values to calculate

int OrientationError = 0;

//Array for storing orientation 
byte dataArrayF[1];           
int arrayIndexF = 0;

// Step counter for autonomous mode
int StepCounter = 0;


void setup() 
{
 
  analogWrite(A0,255);
  analogWrite(A3,0);
  
  dataArrayUL1[arrayIndexUL1] = 100;
  dataArrayUL2[arrayIndexUL2] = 100;
  dataArrayUL3[arrayIndexUL3] = 100;
  dataArrayUR1[arrayIndexUR1] = 100;
  dataArrayUR2[arrayIndexUR2] = 100;
  dataArrayUR3[arrayIndexUR3] = 100;

  // Serial begin for debugging in serial monitor 
  Serial.begin(9600);

  while (!compass.begin())
  {
    Serial.println("Could not find a valid QMC5883 sensor, check wiring!");
    delay(500);
  }

  // Magnometer (digital compass) settings
  Serial.println("Initialize QMC5883");
  compass.setRange(QMC5883_RANGE_2GA);
  compass.setMeasurementMode(QMC5883_CONTINOUS); 
  compass.setDataRate(QMC5883_DATARATE_50HZ);
  compass.setSamples(QMC5883_SAMPLES_8);
}

void loop() 
{ 
  Vector norm = compass.readNormalize();

  // Calculate heading
  float heading = atan2(norm.YAxis, norm.XAxis);

  // Set declination angle on your location and fix heading
  // You can find your declination on: http://magnetic-declination.com/
  // (+) Positive or (-) for negative
  // For Bytom / Poland declination angle is 4'26E (positive)
  // Formula: (deg + (min / 60.0)) / (180 / PI);
  float declinationAngle = (2.0 + (7.0 / 60.0)) / (180 / PI);
  heading += declinationAngle;

  // Correct for heading < 0deg and heading > 360deg
  if (heading < 0)
  {
     heading += 2 * PI;
  }

  if (heading > 2 * PI)
  {
     heading -= 2 * PI;
  }

  // Convert to degrees
  float headingDegrees = heading * 180/PI;
  headingDegrees = map(headingDegrees, 0, 360, 0, 255);
  OrientationError = dataArrayF[arrayIndexF] - headingDegrees;
      if  (OrientationError < -127.5)
      {      
          OrientationError += 255;
      }
      
      if (OrientationError > 127.5)
      {
          OrientationError -= 255;
      }   
      
  if ((millis() >= 14000) && (millis() <= 14250))
  { 
     dataArrayF[arrayIndexF] = headingDegrees;
  }


  if ((millis() >= 14250) && (millis() <= 15000))
  { 
     myservo1.attach(13); 
     myservo1.write(180);
  }

  
  
  if ((millis() > 15000) && (millis() < 35000) && (OrientationError >= 40))          
  {
     myservo1.attach(13); 
     myservo1.write(180);
     StepCounter++;
  }


  if ((millis() > 15000) && (millis() < 35000) && (OrientationError <= -40))
  {
     myservo1.attach(13); 
     myservo1.write(0);  
     StepCounter++;  
  }

  if (millis() >= 35000)
  {
     myservo1.detach();  
  }

  Serial.print(" headingDegrees: ");
  Serial.print(headingDegrees);
  Serial.print(" OrientationError: ");
  Serial.print(OrientationError);
  Serial.println(" ");             
}
